﻿using System;
using System.Collections.Generic;
using Microsoft.Azure.Documents;
using Microsoft.Azure.Documents.Client;
using System.Text;
using HaBookCms.ServiceOptions.Options;
using System.Threading.Tasks;
using System.Net;
using HaBookCms.AzureCosmos.CosmosDB.Interfaces;
using System.Linq;
using System.Reflection;
using Microsoft.Azure.Documents.Linq;
using Newtonsoft.Json;

namespace HaBookCms.AzureCosmos.CosmosDB.Implements
{
    public class AzureCosmosDBRepository: IAzureCosmosDBRepository
    {
        private DocumentClient CosmosClient { get; set; }
        private DocumentCollection CosmosCollection { get; set; }

        private string _Database { get; set; }
        public  AzureCosmosDBRepository(AzureCosmosDBOptions options)
        {
            try {

                CosmosClient = CosmosDBClientSingleton.getInstance(options.ConnectionString, options.ConnectionKey).GetCosmosDBClient();
                _Database = options.Database;
                CosmosClient.CreateDatabaseIfNotExistsAsync(new Database { Id = _Database });
                // _connectionString = options.ConnectionString;
            }
            catch (DocumentClientException de)
            {
                Exception baseException = de.GetBaseException();
                Console.WriteLine("{0} error occurred: {1}, Message: {2}", de.StatusCode, de.Message, baseException.Message);
            }
            catch (Exception e)
            {
                Exception baseException = e.GetBaseException();
                Console.WriteLine("Error: {0}, Message: {1}", e.Message, baseException.Message);
            }
            finally
            {
                Console.WriteLine("End of demo, press any key to exit.");
              //  Console.ReadKey();
            }


        }
        private async Task InitializeCollection<T>()
        {
            Type t = typeof(T);
            if (CosmosCollection == null ||! CosmosCollection.Id.Equals(t.Name)) {
                CosmosCollection = await this.CosmosClient.CreateDocumentCollectionIfNotExistsAsync(UriFactory.CreateDatabaseUri(_Database), new DocumentCollection { Id = t.Name });
               
            }
        }
        public async Task<T> Save<T>(object entity) //where T : object, new()
        {
            Type t = typeof(T);
            await InitializeCollection<T>();
            ResourceResponse<Document> doc = 
                await CosmosClient.CreateDocumentAsync(UriFactory.CreateDocumentCollectionUri(_Database, t.Name), entity);
            Console.WriteLine(doc.ActivityId);
            return (T)entity;
        }

        public async Task<T> Update<T>(object entity) 
        {
            Type t = typeof(T);
            await InitializeCollection<T>();
            ResourceResponse<Document> doc = 
                await CosmosClient.UpsertDocumentAsync(UriFactory.CreateDocumentCollectionUri(_Database, t.Name), entity);
            return (T)entity;
        }

        public async Task<List<T>> FindAll<T>()
        {
            Type t = typeof(T);
            List<T> objs = new List<T>();
            //await InitializeCollection<T>();
            //查询条数 -1是全部
            FeedOptions queryOptions = new FeedOptions { MaxItemCount = -1 };
            var query = CosmosClient.CreateDocumentQuery<T>(UriFactory.CreateDocumentCollectionUri(_Database, t.Name), queryOptions).AsDocumentQuery();
            while (query.HasMoreResults)
            {
                foreach (T obj in await query.ExecuteNextAsync())
                {
                    objs.Add(obj);
                }
            }
            return objs;
            //return CosmosClient.CreateDocumentQuery<T>(UriFactory.CreateDocumentCollectionUri(_Database, t.Name),sql);

        }

        public async Task<List<T>> FindLinq<T>(Func<IQueryable<object>, object> singleOrDefault)
        {
            Type t = typeof(T);
            List<T> objs = new List<T>();
            await InitializeCollection<T>();
            //查询条数 -1是全部
            FeedOptions queryOptions = new FeedOptions { MaxItemCount = -1 };
            var  query = CosmosClient.CreateDocumentQuery<T>(UriFactory.CreateDocumentCollectionUri(_Database, t.Name), queryOptions);
          

            return objs;
            //return CosmosClient.CreateDocumentQuery<T>(UriFactory.CreateDocumentCollectionUri(_Database, t.Name),sql);

        }

        public List<dynamic> FindSQL<dynamic>(string sql)
        {
            Type t = typeof(dynamic);
            List<dynamic> objs = new List<dynamic>();
            //await InitializeCollection<T>();
            var query = CosmosClient.CreateDocumentQuery<dynamic>(UriFactory.CreateDocumentCollectionUri(_Database, t.Name),sql);
            foreach (var item in query)
            {
                objs.Add(item);
            }
            return objs;

        }
    }
}