﻿using Microsoft.IdentityModel.Tokens;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Text;

namespace HaBookCms.Jwt.Model
{
    /// <summary>
    /// JWTToken生成类
    /// </summary>
    public class JwtToken
    {
        public string token_type { get; set; }
        public string token { get; set; }
        public double expires_in { get; set; }
        /// <summary>
        /// 获取基于JWT的Token
        /// </summary>
        /// <param name="claims">需要在登陆的时候配置</param>
        /// <param name="permissionRequirement">在startup中定义的参数</param>
        /// <returns></returns>
        public static JwtToken BuildJwtToken(Claim[] claims
            //, PermissionRequirement permissionRequirement
            )
        {
            var now = DateTime.Now;
            // 实例化JwtSecurityToken
            var jwt = new JwtSecurityToken(
                issuer: BaseConfigModel.Configuration["JwtAuth:Issuer"],
                audience: BaseConfigModel.Configuration["JwtAuth:Audience"],
                claims: claims,
                notBefore: now,
                expires: now.Add(TimeSpan.FromSeconds(60 * 2)),
                signingCredentials: new SigningCredentials(new SymmetricSecurityKey(Encoding.ASCII.GetBytes(BaseConfigModel.Configuration["JwtAuth:SecurityKey"])), SecurityAlgorithms.HmacSha256)
            );
            // 生成 Token
            var encodedJwt = new JwtSecurityTokenHandler().WriteToken(jwt);

            //打包返回前台
            var responseJson = new
            JwtToken
            {
                token = encodedJwt,
                expires_in = TimeSpan.FromSeconds(60 * 2).TotalSeconds,
                token_type = "Bearer"
            };
            return responseJson;
        }
    }
}
